@@ -31,6 +31,10 @@
 /*	char	*arg;
 /*	ssize_t	arg_len;
 /*
+/*	ARGV	*argv_addv(argvp, argv)
+/*	ARGV	*argvp;
+/*	const char **argv;
+/*
 /*	void	argv_terminate(argvp);
 /*	ARGV	*argvp;
 /*
@@ -94,6 +98,10 @@
 /*	argv_addn() is like argv_add(), but each string is followed
 /*	by a string length argument.
 /*
+/*	argv_addv() optionally creates an ARGV when the first argument
+/*	is a null pointer, and appends a null-terminated list of
+/*	strings. The result is null terminated.
+/*
 /*	argv_free() releases storage for a string array, and conveniently
 /*	returns a null pointer.
 /*
@@ -144,6 +152,9 @@
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System libraries. */
@@ -301,6 +312,23 @@ void    argv_addn(ARGV *argvp,...)
     argvp->argv[argvp->argc] = 0;
 }
 
+/* argv_addv - optionally create ARGV, append string vector */
+
+ARGV   *argv_addv(ARGV *argvp, const char *const * argv)
+{
+    const char *const * cpp;
+
+    if (argvp == 0) {
+	for (cpp = argv; *cpp; cpp++)
+	     /* void */ ;
+	argvp = argv_alloc(cpp - argv);
+    }
+    for (cpp = argv; *cpp; cpp++)
+	argv_add(argvp, *cpp, (char *) 0);
+    argvp->argv[argvp->argc] = 0;
+    return (argvp);
+}
+
 /* argv_terminate - terminate string array */
 
 void    argv_terminate(ARGV *argvp)
@@ -602,6 +630,23 @@ static ARGV *test_argv_join(const TEST_CASE *tp, ARGV *argvp)
     return (argvp);
 }
 
+/* test_argv_addv_appends - populate result */
+
+static ARGV *test_argv_addv_appends(const TEST_CASE *tp, ARGV *argvp)
+{
+    argvp = argv_addv(argvp, tp->inputs);
+    return (argvp);
+}
+
+/* test_argv_addv_creates_appends - populate result */
+
+static ARGV *test_argv_addv_creates(const TEST_CASE *tp, ARGV *argvp)
+{
+    argv_free(argvp);
+    argvp = argv_addv((ARGV *) 0, tp->inputs);
+    return (argvp);
+}
+
 /* test_argv_verify - verify result */
 
 static int test_argv_verify(const TEST_CASE *tp, ARGV *argvp)
@@ -737,6 +782,14 @@ static const TEST_CASE test_cases[] = {
 	{0}, 0, test_argv_join,
 	0, 1, {"", 0}, ':'
     },
+    {"argv_addv appends to ARGV",
+	{"foo", "baz", "bar", 0}, /* ignored */ 0, test_argv_addv_appends,
+	0, 3, {"foo", "baz", "bar", 0}
+    },
+    {"argv_addv creates ARGV",
+	{"foo", "baz", "bar", 0}, /* ignored */ 0, test_argv_addv_creates,
+	0, 3, {"foo", "baz", "bar", 0}
+    },
     0,
 };
 
@@ -754,7 +807,7 @@ int     main(int argc, char **argv)
 
 	argvp = argv_alloc(1);
 	if (setjmp(test_panic_jbuf) == 0)
-	    tp->populate_fn(tp, argvp);
+	    argvp = tp->populate_fn(tp, argvp);
 	test_failed = test_argv_verify(tp, argvp);
 	if (test_failed) {
 	    msg_info("%s: FAIL", tp->label);
